@@ -122,48 +122,62 @@ namespace zypp
       }
 
       void
-      setQueryStringMap(const zypp::url::ParamMap &pmap) override
+      setQueryStringMap(const zypp::url::ParamMap &pmap, EEncoding eflag ) override
       {
         static const char * const keys[] = {
           "attrs", "scope", "filter", "exts", NULL
         };
 
-        // remove psep ("?") from safe chars
-        std::string join_safe;
-        std::string safe(config("safe_querystr"));
-        std::string psep(config("psep_querystr"));
-        for(std::string::size_type i=0; i<safe.size(); i++)
+        if ( eflag == url::E_DECODED )
         {
-          if( psep.find(safe[i]) == std::string::npos)
-            join_safe.append(1, safe[i]);
-        }
-
-        zypp::url::ParamVec pvec(4);
-        zypp::url::ParamMap::const_iterator p;
-        for(p=pmap.begin(); p!=pmap.end(); ++p)
-        {
-          bool found=false;
-          for(size_t i=0; i<4; i++)
+          // remove psep ("?") from safe chars
+          std::string join_safe;
+          std::string safe(config("safe_querystr"));
+          std::string psep(config("psep_querystr"));
+          for(std::string::size_type i=0; i<safe.size(); i++)
           {
-            if(p->first == keys[i])
+            if( psep.find(safe[i]) == std::string::npos)
+              join_safe.append(1, safe[i]);
+          }
+
+          zypp::url::ParamVec pvec(4);
+          zypp::url::ParamMap::const_iterator p;
+          for(p=pmap.begin(); p!=pmap.end(); ++p)
+          {
+            bool found=false;
+            for(size_t i=0; i<4; i++)
             {
-              found=true;
-              pvec[i] = zypp::url::encode(p->second, join_safe);
+              if(p->first == keys[i])
+              {
+                found=true;
+                pvec[i] = zypp::url::encode(p->second, join_safe);
+              }
+            }
+            if( !found)
+            {
+              ZYPP_THROW(url::UrlNotSupportedException(
+                str::form(_("Invalid LDAP URL query parameter '%s'"),
+                          p->first.c_str())
+              ));
             }
           }
-          if( !found)
-          {
-            ZYPP_THROW(url::UrlNotSupportedException(
-              str::form(_("Invalid LDAP URL query parameter '%s'"),
-                          p->first.c_str())
-            ));
-          }
+          setQueryStringVec(pvec);
+        }
+        else
+        {
+          setQueryString(
+            zypp::url::join(
+              pmap,
+              config("psep_querystr"),
+              config("vsep_querystr"),
+              config("safe_querystr"),
+              url::E_ENCODED
+            )
+          );
         }
-        setQueryStringVec(pvec);
       }
     };
 
-
     // ---------------------------------------------------------------
     // FIXME: hmm..
     class UrlByScheme
@@ -788,7 +802,45 @@ namespace zypp
   // -----------------------------------------------------------------
 
   void Url::appendPathName( const Pathname & path_r, EEncoding eflag_r )
-  { if ( ! path_r.emptyOrRoot() ) setPathName( Pathname(getPathName( eflag_r )) / path_r, eflag_r ); }
+  {
+    if ( ! path_r.emptyOrRoot() ) {
+      // Check on string level to restore leading double slashes "//" (E_DECODED)
+      // Most important for ftp: where "/%2f" (E_ENCODED) denotes an absolute path.
+      std::string upath { getPathName( url::E_DECODED ) };
+      if ( upath.empty() ) {
+        setPathName( path_r.absolutename(), eflag_r );
+      } else {
+        bool doubleslhash = str::startsWith( upath, "//" );
+        Pathname npath { upath };   // now let Pathname handle the correct concatenation
+        if ( eflag_r == url::E_DECODED ) {
+          npath /= path_r;
+        } else {
+          npath /= url::decode( path_r.asString() );
+        }
+        if ( doubleslhash ) {
+          setPathName( "/" + npath.asString(), url::E_DECODED );
+        } else {
+          setPathName( npath.asString(), url::E_DECODED );
+        }
+      }
+    }
+  }
+
+  void Url::pathNameSetTrailingSlash( bool apply_r )
+  {
+    std::string upath { getPathName( url::E_DECODED ) };
+    if ( upath.empty() || upath == "/" || upath == "//" )
+      return;
+    if ( str::endsWith( upath, "/" ) == apply_r )
+      return;
+
+    if ( apply_r ) {
+      setPathName( upath+"/", url::E_DECODED );
+    } else {
+      do { upath.pop_back(); } while ( str::endsWith( upath, "/" ) );
+      setPathName( upath, url::E_DECODED );
+    }
+  }
 
   // -----------------------------------------------------------------
   void
@@ -834,7 +886,7 @@ namespace zypp
   void
   Url::setQueryStringMap(const zypp::url::ParamMap &pmap)
   {
-    m_impl->setQueryStringMap(pmap);
+    m_impl->setQueryStringMap(pmap, url::E_DECODED);
   }
 
   // -----------------------------------------------------------------
@@ -851,6 +903,12 @@ namespace zypp
     m_impl->delQueryParam(param);
   }
 
+  void
+  Url::delQueryParams(const std::set<std::string> &params)
+  {
+    m_impl->delQueryParams(params);
+  }
+
   // -----------------------------------------------------------------
   ViewOptions
   Url::getViewOptions() const
