@@ -40,7 +40,7 @@ extern "C"
 #include <zypp/Date.h>
 #include <zypp/Pathname.h>
 #include <zypp/PathInfo.h>
-#include <zypp/PublicKey.h>
+#include <zypp-common/PublicKey.h>
 #include <zypp-core/ui/ProgressData>
 
 #include <zypp/target/rpm/RpmDb.h>
@@ -52,7 +52,7 @@ extern "C"
 #include <zypp/target/rpm/RpmException.h>
 #include <zypp/TmpPath.h>
 #include <zypp/KeyRing.h>
-#include <zypp/KeyManager.h>
+#include <zypp-common/KeyManager.h>
 #include <zypp/ZYppFactory.h>
 #include <zypp/ZConfig.h>
 #include <zypp/base/IOTools.h>
@@ -66,10 +66,6 @@ using namespace zypp::filesystem;
 
 #define WORKAROUNDRPMPWDBUG
 
-// bsc#1216091 indicates that 'rpm --runposttrans' does not execute the
-// scripts chroot if --root is used. We disable it if --root is not /.
-#define WORKAROUNDDUMPPOSTTRANSBUG
-
 #undef ZYPP_BASE_LOGGER_LOGGROUP
 #define ZYPP_BASE_LOGGER_LOGGROUP "librpmDb"
 
@@ -94,9 +90,6 @@ namespace target
 {
 namespace rpm
 {
-  const callback::UserData::ContentType InstallResolvableReport::contentRpmout( "rpmout","installpkg" );
-  const callback::UserData::ContentType RemoveResolvableReport::contentRpmout( "rpmout","removepkg" );
-
 namespace
 {
 const char* quoteInFilename_m = "\'\"";
@@ -1688,7 +1681,7 @@ void RpmDb::doInstallPackage( const Pathname & filename, RpmInstFlags flags, Rpm
 
   // run rpm
   RpmArgVec opts;
-#if defined(WORKAROUNDDUMPPOSTTRANSBUG)
+#if defined(WORKAROUNDDUMPPOSTTRANS_BUG_1216091)
   if ( postTransCollector_r && _root == "/" ) {
 #else
   if ( postTransCollector_r ) {
@@ -1903,7 +1896,7 @@ void RpmDb::doRemovePackage( const std::string & name_r, RpmInstFlags flags, Rpm
 
   // run rpm
   RpmArgVec opts;
-#if defined(WORKAROUNDDUMPPOSTTRANSBUG)
+#if defined(WORKAROUNDDUMPPOSTTRANS_BUG_1216091)
   if ( postTransCollector_r && _root == "/" ) {
 #else
   if ( postTransCollector_r ) {
@@ -2024,7 +2017,7 @@ int RpmDb::runposttrans( const Pathname & filename_r, const std::function<void(c
   // It's a pity, but we need all those verbose debug lines just
   // to figure out which script is currently executed. Otherwise we
   // can't tell which output belongs to which script.
-  static const str::regex rx( "^D: (%.*): scriptlet start$" );
+  static const str::regex rx( "^D: (%.*): (scriptlet start|running .* scriptlet)" );
   str::smatch what;
   std::string line;
   bool silent = true; // discard everything before 1st scriptlet
